#include <avr/io.h>
#include <stdio.h>
#include <avr/interrupt.h>

#include "uartdrv.h"



volatile uint8_t buffer[UART_TX_BUFFER_SIZE+5];
volatile uint8_t bufferReadIdx = 0;
volatile uint8_t bufferWriteIdx = 0;

static inline void enableDataRegisterEmptyInterrupt() {
  UCSRB |= (1 << UDRIE);
}

static inline void disableDataRegisterEmptyInterrupt() {
  UCSRB &= ~(1 << UDRIE);
}

void uartdrvInit() {
  //      TXD
  DDRD |= (1 << PD1);

  //      transmitter enable
  UCSRB = (1 << TXEN);

  //        8 data bit, no parity, 1 stop bit
  UCSRC = (1 << UCSZ1) | (1 << UCSZ0);

  //        2400 Baud @ 16MHz
  UBRRL = 0xa0;
  UBRRH = 0x01;;

  FILE *mystdout = fdevopen(uartdrvPutchar, NULL);
  stdout = mystdout;
}

void uartdrvWrite(uint8_t o) {
  if (bufferWriteIdx == UART_TX_BUFFER_SIZE - 1) {
    while (bufferReadIdx == UART_TX_BUFFER_SIZE);
  } else {
    while (bufferReadIdx == bufferWriteIdx + 1);
  }

  buffer[bufferWriteIdx] = o;
  bufferWriteIdx++;

  enableDataRegisterEmptyInterrupt();
}


int uartdrvPutchar(char c, FILE *stream) {
  if (c == '\n')
    uartdrvPutchar('\r', stream);
  uartdrvWrite((uint8_t) c);
  return 0;
}


ISR(USART_UDRE_vect) {
  if (bufferReadIdx != bufferWriteIdx) {
    UDR = buffer[bufferReadIdx];
    bufferReadIdx++;
    if (bufferReadIdx > UART_TX_BUFFER_SIZE) {
      bufferReadIdx = 0;
    }
  } else {
    disableDataRegisterEmptyInterrupt();
  }
}
